@@ -5,19 +5,26 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "BufferTexture.h"
+
+#include <utility>
+
 #include "libyuv.h"
-#include "mozilla/Move.h"
 #include "mozilla/fallible.h"
 #include "mozilla/gfx/2D.h"
 #include "mozilla/gfx/Logging.h"
 #include "mozilla/layers/CompositableForwarder.h"
 #include "mozilla/layers/ISurfaceAllocator.h"
 #include "mozilla/layers/ImageDataSerializer.h"
+#include "mozilla/layers/TextureForwarder.h"
+
+#include "gfxPlatform.h"
 
 #ifdef MOZ_WIDGET_GTK
 #  include "gfxPlatformGtk.h"
 #endif
 
+using mozilla::ipc::IShmemAllocator;
+
 namespace mozilla {
 namespace layers {
 
@@ -29,7 +36,7 @@ class MemoryTextureData : public BufferTextureData {
                                    LayersBackend aLayersBackend,
                                    TextureFlags aFlags,
                                    TextureAllocationFlags aAllocFlags,
-                                   LayersIPCChannel* aAllocator);
+                                   IShmemAllocator* aAllocator);
 
   virtual TextureData* CreateSimilar(
       LayersIPCChannel* aAllocator, LayersBackend aLayersBackend,
@@ -42,21 +49,31 @@ class MemoryTextureData : public BufferTextureData {
 
   MemoryTextureData(const BufferDescriptor& aDesc,
                     gfx::BackendType aMoz2DBackend, uint8_t* aBuffer,
-                    size_t aBufferSize)
+                    size_t aBufferSize, bool aAutoDeallocate = false)
       : BufferTextureData(aDesc, aMoz2DBackend),
         mBuffer(aBuffer),
-        mBufferSize(aBufferSize) {
+        mBufferSize(aBufferSize),
+        mAutoDeallocate(aAutoDeallocate) {
     MOZ_ASSERT(aBuffer);
     MOZ_ASSERT(aBufferSize);
   }
 
+  virtual ~MemoryTextureData() override {
+    if (mAutoDeallocate) {
+      Deallocate(nullptr);
+    }
+  }
+
   virtual uint8_t* GetBuffer() override { return mBuffer; }
 
   virtual size_t GetBufferSize() override { return mBufferSize; }
 
+  TextureType GetTextureType() const override { return TextureType::Unknown; }
+
  protected:
   uint8_t* mBuffer;
   size_t mBufferSize;
+  bool mAutoDeallocate;
 };
 
 class ShmemTextureData : public BufferTextureData {
@@ -67,7 +84,7 @@ class ShmemTextureData : public BufferTextureData {
                                   LayersBackend aLayersBackend,
                                   TextureFlags aFlags,
                                   TextureAllocationFlags aAllocFlags,
-                                  LayersIPCChannel* aAllocator);
+                                  IShmemAllocator* aAllocator);
 
   virtual TextureData* CreateSimilar(
       LayersIPCChannel* aAllocator, LayersBackend aLayersBackend,
@@ -92,32 +109,12 @@ class ShmemTextureData : public BufferTextureData {
   mozilla::ipc::Shmem mShmem;
 };
 
-static bool UsingX11Compositor() {
-#ifdef MOZ_WIDGET_GTK
-  return gfx::gfxVars::UseXRender();
-#endif
-  return false;
-}
-
-bool ComputeHasIntermediateBuffer(gfx::SurfaceFormat aFormat,
-                                  LayersBackend aLayersBackend,
-                                  bool aSupportsTextureDirectMapping) {
-  if (aSupportsTextureDirectMapping) {
-    return false;
-  }
-
-  return aLayersBackend != LayersBackend::LAYERS_BASIC ||
-         UsingX11Compositor() || aFormat == gfx::SurfaceFormat::UNKNOWN;
-}
-
-BufferTextureData* BufferTextureData::Create(gfx::IntSize aSize,
-                                             gfx::SurfaceFormat aFormat,
-                                             gfx::BackendType aMoz2DBackend,
-                                             LayersBackend aLayersBackend,
-                                             TextureFlags aFlags,
-                                             TextureAllocationFlags aAllocFlags,
-                                             LayersIPCChannel* aAllocator) {
-  if (!aAllocator || aAllocator->IsSameProcess()) {
+BufferTextureData* BufferTextureData::Create(
+    gfx::IntSize aSize, gfx::SurfaceFormat aFormat,
+    gfx::BackendType aMoz2DBackend, LayersBackend aLayersBackend,
+    TextureFlags aFlags, TextureAllocationFlags aAllocFlags,
+    mozilla::ipc::IShmemAllocator* aAllocator, bool aIsSameProcess) {
+  if (!aAllocator || aIsSameProcess) {
     return MemoryTextureData::Create(aSize, aFormat, aMoz2DBackend,
                                      aLayersBackend, aFlags, aAllocFlags,
                                      aAllocator);
@@ -143,7 +140,7 @@ BufferTextureData* BufferTextureData::CreateInternal(
     return new MemoryTextureData(aDesc, aMoz2DBackend, buffer, aBufferSize);
   } else {
     ipc::Shmem shm;
-    if (!aAllocator->AllocUnsafeShmem(aBufferSize, OptimalShmemType(), &shm)) {
+    if (!aAllocator->AllocUnsafeShmem(aBufferSize, &shm)) {
       return nullptr;
     }
 
@@ -152,9 +149,11 @@ BufferTextureData* BufferTextureData::CreateInternal(
 }
 
 BufferTextureData* BufferTextureData::CreateForYCbCr(
-    KnowsCompositor* aAllocator, gfx::IntSize aYSize, uint32_t aYStride,
-    gfx::IntSize aCbCrSize, uint32_t aCbCrStride, StereoMode aStereoMode,
+    KnowsCompositor* aAllocator, const gfx::IntRect& aDisplay,
+    const gfx::IntSize& aYSize, uint32_t aYStride,
+    const gfx::IntSize& aCbCrSize, uint32_t aCbCrStride, StereoMode aStereoMode,
     gfx::ColorDepth aColorDepth, gfx::YUVColorSpace aYUVColorSpace,
+    gfx::ColorRange aColorRange, gfx::ChromaSubsampling aSubsampling,
     TextureFlags aTextureFlags) {
   uint32_t bufSize = ImageDataSerializer::ComputeYCbCrBufferSize(
       aYSize, aYStride, aCbCrSize, aCbCrStride);
@@ -169,23 +168,10 @@ BufferTextureData* BufferTextureData::CreateForYCbCr(
                                            aCbCrSize.height, yOffset, cbOffset,
                                            crOffset);
 
-  bool supportsTextureDirectMapping =
-      aAllocator->SupportsTextureDirectMapping() &&
-      aAllocator->GetMaxTextureSize() >
-          std::max(aYSize.width,
-                   std::max(aYSize.height,
-                            std::max(aCbCrSize.width, aCbCrSize.height)));
-
-  bool hasIntermediateBuffer =
-      aAllocator
-          ? ComputeHasIntermediateBuffer(gfx::SurfaceFormat::YUV,
-                                         aAllocator->GetCompositorBackendType(),
-                                         supportsTextureDirectMapping)
-          : true;
-
-  YCbCrDescriptor descriptor = YCbCrDescriptor(
-      aYSize, aYStride, aCbCrSize, aCbCrStride, yOffset, cbOffset, crOffset,
-      aStereoMode, aColorDepth, aYUVColorSpace, hasIntermediateBuffer);
+  YCbCrDescriptor descriptor =
+      YCbCrDescriptor(aDisplay, aYSize, aYStride, aCbCrSize, aCbCrStride,
+                      yOffset, cbOffset, crOffset, aStereoMode, aColorDepth,
+                      aYUVColorSpace, aColorRange, aSubsampling);
 
   return CreateInternal(
       aAllocator ? aAllocator->GetTextureForwarder() : nullptr, descriptor,
@@ -198,16 +184,8 @@ void BufferTextureData::FillInfo(TextureData::Info& aInfo) const {
   aInfo.hasSynchronization = false;
   aInfo.canExposeMappedData = true;
 
-  if (mDescriptor.type() == BufferDescriptor::TYCbCrDescriptor) {
-    aInfo.hasIntermediateBuffer =
-        mDescriptor.get_YCbCrDescriptor().hasIntermediateBuffer();
-  } else {
-    aInfo.hasIntermediateBuffer =
-        mDescriptor.get_RGBDescriptor().hasIntermediateBuffer();
-  }
-
   switch (aInfo.format) {
-    case gfx::SurfaceFormat::YUV:
+    case gfx::SurfaceFormat::YUV420:
     case gfx::SurfaceFormat::UNKNOWN:
       aInfo.supportsMoz2D = false;
       break;
@@ -220,10 +198,26 @@ gfx::IntSize BufferTextureData::GetSize() const {
   return ImageDataSerializer::SizeFromBufferDescriptor(mDescriptor);
 }
 
+gfx::IntRect BufferTextureData::GetPictureRect() const {
+  return ImageDataSerializer::RectFromBufferDescriptor(mDescriptor);
+}
+
+Maybe<gfx::IntSize> BufferTextureData::GetYSize() const {
+  return ImageDataSerializer::YSizeFromBufferDescriptor(mDescriptor);
+}
+
 Maybe<gfx::IntSize> BufferTextureData::GetCbCrSize() const {
   return ImageDataSerializer::CbCrSizeFromBufferDescriptor(mDescriptor);
 }
 
+Maybe<int32_t> BufferTextureData::GetYStride() const {
+  return ImageDataSerializer::YStrideFromBufferDescriptor(mDescriptor);
+}
+
+Maybe<int32_t> BufferTextureData::GetCbCrStride() const {
+  return ImageDataSerializer::CbCrStrideFromBufferDescriptor(mDescriptor);
+}
+
 Maybe<gfx::YUVColorSpace> BufferTextureData::GetYUVColorSpace() const {
   return ImageDataSerializer::YUVColorSpaceFromBufferDescriptor(mDescriptor);
 }
@@ -236,6 +230,11 @@ Maybe<StereoMode> BufferTextureData::GetStereoMode() const {
   return ImageDataSerializer::StereoModeFromBufferDescriptor(mDescriptor);
 }
 
+Maybe<gfx::ChromaSubsampling> BufferTextureData::GetChromaSubsampling() const {
+  return ImageDataSerializer::ChromaSubsamplingFromBufferDescriptor(
+      mDescriptor);
+}
+
 gfx::SurfaceFormat BufferTextureData::GetFormat() const {
   return ImageDataSerializer::FormatFromBufferDescriptor(mDescriptor);
 }
@@ -269,7 +268,7 @@ already_AddRefed<gfx::DrawTarget> BufferTextureData::BorrowDrawTarget() {
 }
 
 bool BufferTextureData::BorrowMappedData(MappedTextureData& aData) {
-  if (GetFormat() == gfx::SurfaceFormat::YUV) {
+  if (GetFormat() == gfx::SurfaceFormat::YUV420) {
     return false;
   }
 
@@ -381,12 +380,6 @@ bool BufferTextureData::UpdateFromSurface(gfx::SourceSurface* aSurface) {
   return true;
 }
 
-void BufferTextureData::SetDescriptor(BufferDescriptor&& aDescriptor) {
-  MOZ_ASSERT(mDescriptor.type() == BufferDescriptor::TYCbCrDescriptor);
-  MOZ_ASSERT(mDescriptor.get_YCbCrDescriptor().ySize() == gfx::IntSize());
-  mDescriptor = std::move(aDescriptor);
-}
-
 bool MemoryTextureData::Serialize(SurfaceDescriptor& aOutDescriptor) {
   MOZ_ASSERT(GetFormat() != gfx::SurfaceFormat::UNKNOWN);
   if (GetFormat() == gfx::SurfaceFormat::UNKNOWN) {
@@ -407,8 +400,7 @@ static bool InitBuffer(uint8_t* buf, size_t bufSize, gfx::SurfaceFormat aFormat,
     return false;
   }
 
-  if ((aAllocFlags & ALLOC_CLEAR_BUFFER) ||
-      (aAllocFlags & ALLOC_CLEAR_BUFFER_BLACK)) {
+  if (aAllocFlags & ALLOC_CLEAR_BUFFER) {
     if (aFormat == gfx::SurfaceFormat::B8G8R8X8) {
       // Even though BGRX was requested, XRGB_UINT32 is what is meant,
       // so use 0xFF000000 to put alpha in the right place.
@@ -419,10 +411,6 @@ static bool InitBuffer(uint8_t* buf, size_t bufSize, gfx::SurfaceFormat aFormat,
     }
   }
 
-  if (aAllocFlags & ALLOC_CLEAR_BUFFER_WHITE) {
-    memset(buf, 0xFF, bufSize);
-  }
-
   return true;
 }
 
@@ -432,9 +420,9 @@ MemoryTextureData* MemoryTextureData::Create(gfx::IntSize aSize,
                                              LayersBackend aLayersBackend,
                                              TextureFlags aFlags,
                                              TextureAllocationFlags aAllocFlags,
-                                             LayersIPCChannel* aAllocator) {
+                                             IShmemAllocator* aAllocator) {
   // Should have used CreateForYCbCr.
-  MOZ_ASSERT(aFormat != gfx::SurfaceFormat::YUV);
+  MOZ_ASSERT(aFormat != gfx::SurfaceFormat::YUV420);
 
   if (aSize.width <= 0 || aSize.height <= 0) {
     gfxDebug() << "Asking for buffer of invalid size " << aSize.width << "x"
@@ -452,15 +440,15 @@ MemoryTextureData* MemoryTextureData::Create(gfx::IntSize aSize,
     return nullptr;
   }
 
-  bool hasIntermediateBuffer = ComputeHasIntermediateBuffer(
-      aFormat, aLayersBackend, aAllocFlags & ALLOC_ALLOW_DIRECT_MAPPING);
-
   GfxMemoryImageReporter::DidAlloc(buf);
 
-  BufferDescriptor descriptor =
-      RGBDescriptor(aSize, aFormat, hasIntermediateBuffer);
+  BufferDescriptor descriptor = RGBDescriptor(aSize, aFormat);
 
-  return new MemoryTextureData(descriptor, aMoz2DBackend, buf, bufSize);
+  // Remote textures are not managed by a texture client, so we need to ensure
+  // that memory is freed when the owning MemoryTextureData goes away.
+  bool autoDeallocate = !!(aFlags & TextureFlags::REMOTE_TEXTURE);
+  return new MemoryTextureData(descriptor, aMoz2DBackend, buf, bufSize,
+                               autoDeallocate);
 }
 
 void MemoryTextureData::Deallocate(LayersIPCChannel*) {
@@ -496,10 +484,10 @@ ShmemTextureData* ShmemTextureData::Create(gfx::IntSize aSize,
                                            LayersBackend aLayersBackend,
                                            TextureFlags aFlags,
                                            TextureAllocationFlags aAllocFlags,
-                                           LayersIPCChannel* aAllocator) {
+                                           IShmemAllocator* aAllocator) {
   MOZ_ASSERT(aAllocator);
   // Should have used CreateForYCbCr.
-  MOZ_ASSERT(aFormat != gfx::SurfaceFormat::YUV);
+  MOZ_ASSERT(aFormat != gfx::SurfaceFormat::YUV420);
 
   if (!aAllocator) {
     return nullptr;
@@ -517,7 +505,7 @@ ShmemTextureData* ShmemTextureData::Create(gfx::IntSize aSize,
   }
 
   mozilla::ipc::Shmem shm;
-  if (!aAllocator->AllocUnsafeShmem(bufSize, OptimalShmemType(), &shm)) {
+  if (!aAllocator->AllocUnsafeShmem(bufSize, &shm)) {
     return nullptr;
   }
 
@@ -526,11 +514,7 @@ ShmemTextureData* ShmemTextureData::Create(gfx::IntSize aSize,
     return nullptr;
   }
 
-  bool hasIntermediateBuffer = ComputeHasIntermediateBuffer(
-      aFormat, aLayersBackend, aAllocFlags & ALLOC_ALLOW_DIRECT_MAPPING);
-
-  BufferDescriptor descriptor =
-      RGBDescriptor(aSize, aFormat, hasIntermediateBuffer);
+  BufferDescriptor descriptor = RGBDescriptor(aSize, aFormat);
 
   return new ShmemTextureData(descriptor, aMoz2DBackend, shm);
 }
@@ -544,6 +528,10 @@ TextureData* ShmemTextureData::CreateSimilar(
 }
 
 void ShmemTextureData::Deallocate(LayersIPCChannel* aAllocator) {
+  if (!aAllocator) {
+    gfxCriticalNote << "No allocator in ShmemTextureData::Deallocate";
+    return;
+  }
   aAllocator->DeallocShmem(mShmem);
 }
 
