@@ -42,10 +42,10 @@ class APZEventResultTester : public APZCTreeManagerTester {
 
   void CreateScrollableRootLayer() {
     const char* treeShape = "x";
-    LayerIntRegion layerVisibleRegions[] = {
+    LayerIntRect layerVisibleRects[] = {
         LayerIntRect(0, 0, 100, 100),
     };
-    CreateScrollData(treeShape, layerVisibleRegions);
+    CreateScrollData(treeShape, layerVisibleRects);
     SetScrollableFrameMetrics(root, ScrollableLayerGuid::START_SCROLL_ID,
                               CSSRect(0, 0, 200, 200));
     ModifyFrameMetrics(root, [](ScrollMetadata& sm, FrameMetrics& metrics) {
@@ -64,11 +64,10 @@ class APZEventResultTester : public APZCTreeManagerTester {
     APZHandledResult delayedAnswer{APZHandledPlace::Invalid, SideBits::eNone,
                                    ScrollDirections()};
     manager->AddInputBlockCallback(
-        result.mInputBlockId,
-        {result.GetStatus(), [&](uint64_t id, const APZHandledResult& answer) {
-           EXPECT_EQ(id, result.mInputBlockId);
-           delayedAnswer = answer;
-         }});
+        result.mInputBlockId, [&](uint64_t id, const APZHandledResult& answer) {
+          EXPECT_EQ(id, result.mInputBlockId);
+          delayedAnswer = answer;
+        });
     manager->SetAllowedTouchBehavior(result.mInputBlockId,
                                      {AllowedTouchBehavior::VERTICAL_PAN});
     manager->SetTargetAPZC(result.mInputBlockId, {result.mTargetGuid});
@@ -357,10 +356,10 @@ TEST_F(APZEventResultTesterMock, HandledByRootApzcFlag) {
   // Create simple layer tree containing a dispatch-to-content region
   // that covers part but not all of its area.
   const char* treeShape = "x";
-  LayerIntRegion layerVisibleRegions[] = {
+  LayerIntRect layerVisibleRects[] = {
       LayerIntRect(0, 0, 100, 100),
   };
-  CreateScrollData(treeShape, layerVisibleRegions);
+  CreateScrollData(treeShape, layerVisibleRects);
   SetScrollableFrameMetrics(root, ScrollableLayerGuid::START_SCROLL_ID,
                             CSSRect(0, 0, 100, 200));
   ModifyFrameMetrics(root, [](ScrollMetadata& sm, FrameMetrics& metrics) {
@@ -394,11 +393,10 @@ TEST_F(APZEventResultTesterMock, HandledByRootApzcFlag) {
   APZHandledResult delayedAnswer{APZHandledPlace::Invalid, SideBits::eNone,
                                  ScrollDirections()};
   manager->AddInputBlockCallback(
-      result.mInputBlockId,
-      {result.GetStatus(), [&](uint64_t id, const APZHandledResult& answer) {
-         EXPECT_EQ(id, result.mInputBlockId);
-         delayedAnswer = answer;
-       }});
+      result.mInputBlockId, [&](uint64_t id, const APZHandledResult& answer) {
+        EXPECT_EQ(id, result.mInputBlockId);
+        delayedAnswer = answer;
+      });
 
   // Send APZ the relevant notifications to allow it to process the
   // input block.
@@ -422,11 +420,10 @@ TEST_F(APZEventResultTesterMock, HandledByRootApzcFlag) {
   TouchUp(manager, ScreenIntPoint(50, 75), mcc->Time());
   EXPECT_EQ(result.GetHandledResult(), Nothing());
   manager->AddInputBlockCallback(
-      result.mInputBlockId,
-      {result.GetStatus(), [&](uint64_t id, const APZHandledResult& answer) {
-         EXPECT_EQ(id, result.mInputBlockId);
-         delayedAnswer = answer;
-       }});
+      result.mInputBlockId, [&](uint64_t id, const APZHandledResult& answer) {
+        EXPECT_EQ(id, result.mInputBlockId);
+        delayedAnswer = answer;
+      });
   manager->SetAllowedTouchBehavior(result.mInputBlockId,
                                    {AllowedTouchBehavior::VERTICAL_PAN});
   manager->SetTargetAPZC(result.mInputBlockId, {result.mTargetGuid});
@@ -450,11 +447,10 @@ TEST_F(APZEventResultTesterMock, HandledByRootApzcFlag) {
   TouchUp(manager, ScreenIntPoint(50, 75), mcc->Time());
   EXPECT_EQ(result.GetHandledResult(), Nothing());
   manager->AddInputBlockCallback(
-      result.mInputBlockId,
-      {result.GetStatus(), [&](uint64_t id, const APZHandledResult& answer) {
-         EXPECT_EQ(id, result.mInputBlockId);
-         delayedAnswer = answer;
-       }});
+      result.mInputBlockId, [&](uint64_t id, const APZHandledResult& answer) {
+        EXPECT_EQ(id, result.mInputBlockId);
+        delayedAnswer = answer;
+      });
   manager->SetAllowedTouchBehavior(result.mInputBlockId,
                                    {AllowedTouchBehavior::VERTICAL_PAN});
   manager->SetTargetAPZC(result.mInputBlockId, {result.mTargetGuid});
@@ -462,7 +458,7 @@ TEST_F(APZEventResultTesterMock, HandledByRootApzcFlag) {
                                      /*aPreventDefault=*/false);
   EXPECT_EQ(delayedAnswer,
             (APZHandledResult{APZHandledPlace::Unhandled, SideBits::eNone,
-                              ScrollDirections()}));
+                              EitherScrollDirection}));
 
   // Repeat the tap on the bottom half, with no event handler.
   // Make sure we get an eager answer of `Unhandled`.
