@@ -26,7 +26,7 @@ void HostIPCAllocator::SendPendingAsyncMessages() {
   // Some type of AsyncParentMessageData message could have
   // one file descriptor (e.g. OpDeliverFence).
   // A number of file descriptors per gecko ipc message have a limitation
-  // on OS_POSIX (MACOSX or LINUX).
+  // on XP_UNIX (MACOSX or LINUX).
   static const uint32_t kMaxMessageNumber =
       IPC::Message::MAX_DESCRIPTORS_PER_MESSAGE;
 
@@ -74,6 +74,7 @@ bool FixedSizeSmallShmemSectionAllocator::AllocShmemSection(
     uint32_t aSize, ShmemSection* aShmemSection) {
   // For now we only support sizes of 4. If we want to support different sizes
   // some more complicated bookkeeping should be added.
+  NS_ASSERT_OWNINGTHREAD(FixedSizeSmallShmemSectionAllocator);
   MOZ_ASSERT(aSize == sSupportedBlockSize);
   MOZ_ASSERT(aShmemSection);
 
@@ -184,6 +185,8 @@ void FixedSizeSmallShmemSectionAllocator::FreeShmemSection(
 
 void FixedSizeSmallShmemSectionAllocator::DeallocShmemSection(
     mozilla::layers::ShmemSection& aShmemSection) {
+  NS_ASSERT_OWNINGTHREAD(FixedSizeSmallShmemSectionAllocator);
+
   if (!IPCOpen()) {
     gfxCriticalNote << "Attempt to dealloc a ShmemSections after shutdown.";
     return;
@@ -194,6 +197,8 @@ void FixedSizeSmallShmemSectionAllocator::DeallocShmemSection(
 }
 
 void FixedSizeSmallShmemSectionAllocator::ShrinkShmemSectionHeap() {
+  NS_ASSERT_OWNINGTHREAD(FixedSizeSmallShmemSectionAllocator);
+
   if (!IPCOpen()) {
     mUsedShmems.clear();
     return;
