@@ -11,10 +11,13 @@
 #include "src/core/SkAdvancedTypefaceMetrics.h"
 #include "src/core/SkFDot6.h"
 #include "include/core/SkFontMetrics.h"
+#include "include/core/SkFontTypes.h"
 #include "include/core/SkPath.h"
+#include "include/core/SkStream.h"
 #include "src/core/SkScalerContext.h"
 #include "src/core/SkTypefaceCache.h"
 
+#include <cfloat>
 #include <cmath>
 
 #include <ft2build.h>
@@ -95,9 +98,9 @@ void SkInitCairoFT(bool fontHintingEnabled)
     }
 }
 
-class SkScalerContext_CairoFT : public SkScalerContext_FreeType_Base {
+class SkScalerContext_CairoFT : public SkScalerContext {
 public:
-    SkScalerContext_CairoFT(sk_sp<SkTypeface> typeface,
+    SkScalerContext_CairoFT(SkTypeface& typeface,
                             const SkScalerContextEffects& effects,
                             const SkDescriptor* desc, FT_Face face,
                             void* faceContext, SkPixelGeometry pixelGeometry,
@@ -120,16 +123,16 @@ public:
     void Unlock() { mozilla_UnlockSharedFTFace(fFTFaceContext); }
 
 protected:
-    bool generateAdvance(SkGlyph* glyph) override;
-    void generateMetrics(SkGlyph* glyph, SkArenaAlloc* arena) override;
-    void generateImage(const SkGlyph& glyph) override;
-    bool generatePath(const SkGlyph& glyph, SkPath* path) override;
+    GlyphMetrics generateMetrics(const SkGlyph& glyph, SkArenaAlloc* arena) override;
+    void generateImage(const SkGlyph& glyph, void* imageBuffer) override;
+    bool generatePath(const SkGlyph& glyph, SkPath* path, bool* modified) override;
     void generateFontMetrics(SkFontMetrics* metrics) override;
 
 private:
     bool computeShapeMatrix(const SkMatrix& m);
-    void prepareGlyph(FT_GlyphSlot glyph);
+    bool prepareGlyph(FT_GlyphSlot glyph);
 
+    SkScalerContextFTUtils fUtils;
     FT_Face fFTFace;
     void* fFTFaceContext;
     FT_Int32 fLoadGlyphFlags;
@@ -182,7 +185,7 @@ public:
     std::unique_ptr<SkScalerContext> onCreateScalerContext(const SkScalerContextEffects& effects, const SkDescriptor* desc) const override
     {
         SkScalerContext_CairoFT* ctx = new SkScalerContext_CairoFT(
-            sk_ref_sp(const_cast<SkCairoFTTypeface*>(this)), effects, desc,
+            *const_cast<SkCairoFTTypeface*>(this), effects, desc,
             fFTFace, fFTFaceContext, fPixelGeometry, fLcdFilter);
         std::unique_ptr<SkScalerContext> result(ctx);
         if (!ctx->isValid()) {
@@ -329,10 +332,10 @@ SkTypeface* SkCreateTypefaceFromCairoFTFont(FT_Face face, void* faceContext,
 }
 
 SkScalerContext_CairoFT::SkScalerContext_CairoFT(
-    sk_sp<SkTypeface> typeface, const SkScalerContextEffects& effects,
+    SkTypeface& typeface, const SkScalerContextEffects& effects,
     const SkDescriptor* desc, FT_Face face, void* faceContext,
     SkPixelGeometry pixelGeometry, FT_LcdFilter lcdFilter)
-    : SkScalerContext_FreeType_Base(std::move(typeface), effects, desc)
+    : SkScalerContext(typeface, effects, desc)
     , fFTFace(face)
     , fFTFaceContext(faceContext)
     , fLcdFilter(lcdFilter)
@@ -395,7 +398,7 @@ SkScalerContext_CairoFT::SkScalerContext_CairoFT(
             }
             break;
         default:
-            SkDebugf("---------- UNKNOWN hinting %d\n", fRec.getHinting());
+            SkDebugf("---------- UNKNOWN hinting %d\n", (int)fRec.getHinting());
             break;
         }
     }
@@ -419,6 +422,8 @@ SkScalerContext_CairoFT::SkScalerContext_CairoFT(
     loadFlags |= FT_LOAD_COLOR;
 
     fLoadGlyphFlags = loadFlags;
+
+    fUtils.init(fRec.fForegroundColor, (SkScalerContext::Flags)fRec.fFlags);
 }
 
 bool SkScalerContext_CairoFT::computeShapeMatrix(const SkMatrix& m)
@@ -485,50 +490,45 @@ bool SkScalerContext_CairoFT::computeShapeMatrix(const SkMatrix& m)
     return true;
 }
 
-bool SkScalerContext_CairoFT::generateAdvance(SkGlyph* glyph)
-{
-    generateMetrics(glyph, nullptr);
-    return !glyph->isEmpty();
-}
-
-void SkScalerContext_CairoFT::prepareGlyph(FT_GlyphSlot glyph)
+bool SkScalerContext_CairoFT::prepareGlyph(FT_GlyphSlot glyph)
 {
+    bool modified = false;
     if (fRec.fFlags & SkScalerContext::kEmbolden_Flag) {
         // Not FT_GlyphSlot_Embolden because we want a less extreme effect.
         mozilla_glyphslot_embolden_less(glyph);
+        modified = true;
     }
+    return modified;
 }
 
-void SkScalerContext_CairoFT::generateMetrics(SkGlyph* glyph, SkArenaAlloc* arena)
+SkScalerContext::GlyphMetrics SkScalerContext_CairoFT::generateMetrics(const SkGlyph& glyph, SkArenaAlloc* arena)
 {
-    glyph->fMaskFormat = fRec.fMaskFormat;
-
-    glyph->zeroMetrics();
+    GlyphMetrics mx(glyph.maskFormat());
 
     AutoLockFTFace faceLock(this);
 
-    FT_Error err = mozilla_LoadFTGlyph(fFTFace, glyph->getGlyphID(), fLoadGlyphFlags);
+    FT_Error err = mozilla_LoadFTGlyph(fFTFace, glyph.getGlyphID(), fLoadGlyphFlags);
     if (err != 0) {
-        return;
+        return mx;
     }
 
     prepareGlyph(fFTFace->glyph);
 
-    glyph->fAdvanceX = SkFDot6ToFloat(fFTFace->glyph->advance.x);
-    glyph->fAdvanceY = -SkFDot6ToFloat(fFTFace->glyph->advance.y);
+    mx.advance.fX = SkFDot6ToFloat(fFTFace->glyph->advance.x);
+    mx.advance.fY = -SkFDot6ToFloat(fFTFace->glyph->advance.y);
 
     SkIRect bounds;
     switch (fFTFace->glyph->format) {
     case FT_GLYPH_FORMAT_OUTLINE:
         if (!fFTFace->glyph->outline.n_contours) {
-            return;
+            return mx;
         }
 
         FT_BBox bbox;
         FT_Outline_Get_CBox(&fFTFace->glyph->outline, &bbox);
         if (this->isSubpixel()) {
-            int dx = SkFixedToFDot6(glyph->getSubXFixed());
-            int dy = SkFixedToFDot6(glyph->getSubYFixed());
+            int dx = SkFixedToFDot6(glyph.getSubXFixed());
+            int dy = SkFixedToFDot6(glyph.getSubYFixed());
             bbox.xMin += dx;
             bbox.yMin -= dy;
             bbox.xMax += dx;
@@ -560,8 +560,10 @@ void SkScalerContext_CairoFT::generateMetrics(SkGlyph* glyph, SkArenaAlloc* aren
         }
         break;
     case FT_GLYPH_FORMAT_BITMAP:
+        mx.neverRequestPath = true;
+
         if (fFTFace->glyph->bitmap.pixel_mode == FT_PIXEL_MODE_BGRA) {
-            glyph->fMaskFormat = SkMask::kARGB32_Format;
+            mx.maskFormat = SkMask::kARGB32_Format;
         }
 
         if (isLCD(fRec)) {
@@ -601,25 +603,24 @@ void SkScalerContext_CairoFT::generateMetrics(SkGlyph* glyph, SkArenaAlloc* aren
         break;
     default:
         SkDEBUGFAIL("unknown glyph format");
-        return;
+        return mx;
     }
 
     if (SkIRect::MakeXYWH(SHRT_MIN, SHRT_MIN, USHRT_MAX, USHRT_MAX).contains(bounds)) {
-        glyph->fWidth  = SkToU16(bounds.width());
-        glyph->fHeight = SkToU16(bounds.height());
-        glyph->fLeft   = SkToS16(bounds.left());
-        glyph->fTop    = SkToS16(bounds.top());
+      mx.bounds = SkRect::Make(bounds);
     }
+
+    return mx;
 }
 
-void SkScalerContext_CairoFT::generateImage(const SkGlyph& glyph)
+void SkScalerContext_CairoFT::generateImage(const SkGlyph& glyph, void* imageBuffer)
 {
     AutoLockFTFace faceLock(this);
 
     FT_Error err = mozilla_LoadFTGlyph(fFTFace, glyph.getGlyphID(), fLoadGlyphFlags);
 
     if (err != 0) {
-        memset(glyph.fImage, 0, glyph.rowBytes() * glyph.fHeight);
+        sk_bzero(imageBuffer, glyph.imageSize());
         return;
     }
 
@@ -641,7 +642,7 @@ void SkScalerContext_CairoFT::generateImage(const SkGlyph& glyph)
     } else {
         matrix.setIdentity();
     }
-    generateGlyphImage(fFTFace, glyph, matrix);
+    fUtils.generateGlyphImage(fFTFace, glyph, imageBuffer, matrix, fPreBlend);
 
     if (useLcdFilter) {
         gSetLcdFilter(fFTFace->glyph->library, FT_LCD_FILTER_NONE);
@@ -649,7 +650,7 @@ void SkScalerContext_CairoFT::generateImage(const SkGlyph& glyph)
     }
 }
 
-bool SkScalerContext_CairoFT::generatePath(const SkGlyph& glyph, SkPath* path)
+bool SkScalerContext_CairoFT::generatePath(const SkGlyph& glyph, SkPath* path, bool* modified)
 {
     AutoLockFTFace faceLock(this);
 
@@ -668,14 +669,14 @@ bool SkScalerContext_CairoFT::generatePath(const SkGlyph& glyph, SkPath* path)
         return false;
     }
 
-    prepareGlyph(fFTFace->glyph);
+    *modified |= prepareGlyph(fFTFace->glyph);
 
-    return generateGlyphPath(fFTFace, path);
+    return fUtils.generateGlyphPath(fFTFace, path);
 }
 
 void SkScalerContext_CairoFT::generateFontMetrics(SkFontMetrics* metrics)
 {
     if (metrics) {
-        memset(metrics, 0, sizeof(SkFontMetrics));
+        sk_bzero(metrics, sizeof(SkFontMetrics));
     }
 }
