@@ -71,6 +71,40 @@ ALWAYS_INLINE uint8_t convert_pixel<uint8_t>(uint16_t src) {
 #endif
 }
 
+// Apply a u8 alpha mask to a u32 texture row
+static inline void mask_row(uint32_t* dst, const uint8_t* mask, int span) {
+  auto* end = dst + span;
+  while (dst + 4 <= end) {
+    WideRGBA8 maskpx = expand_mask(dst, unpack(unaligned_load<PackedR8>(mask)));
+    WideRGBA8 dstpx = unpack(unaligned_load<PackedRGBA8>(dst));
+    PackedRGBA8 r = pack(muldiv255(dstpx, maskpx));
+    unaligned_store(dst, r);
+    mask += 4;
+    dst += 4;
+  }
+  if (dst < end) {
+    WideRGBA8 maskpx = expand_mask(dst, unpack(partial_load_span<PackedR8>(mask, end - dst)));
+    WideRGBA8 dstpx = unpack(partial_load_span<PackedRGBA8>(dst, end - dst));
+    auto r = pack(maskpx + dstpx - muldiv255(dstpx, maskpx));
+    partial_store_span(dst, r, end - dst);
+  }
+}
+
+// Apply a R8 alpha mask to a RGBA8 texture
+static NO_INLINE void mask_blit(Texture& masktex, Texture& dsttex) {
+  int maskStride = masktex.stride();
+  int destStride = dsttex.stride();
+  char* dest = dsttex.sample_ptr(0, 0);
+  char* mask = masktex.sample_ptr(0, 0);
+  int span = dsttex.width;
+
+  for (int rows = dsttex.height; rows > 0; rows--) {
+      mask_row((uint32_t*)dest, (uint8_t*)mask, span);
+      dest += destStride;
+      mask += maskStride;
+  }
+}
+
 template <bool COMPOSITE, typename P>
 static inline void copy_row(P* dst, const P* src, int span) {
   // No scaling, so just do a fast copy.
@@ -479,52 +513,6 @@ void BlitFramebuffer(GLint srcX0, GLint srcY0, GLint srcX1, GLint srcY1,
   }
 }
 
-typedef Texture LockedTexture;
-
-// Lock the given texture to prevent modification.
-LockedTexture* LockTexture(GLuint texId) {
-  Texture& tex = ctx->textures[texId];
-  if (!tex.buf) {
-    assert(tex.buf != nullptr);
-    return nullptr;
-  }
-  if (__sync_fetch_and_add(&tex.locked, 1) == 0) {
-    // If this is the first time locking the texture, flush any delayed clears.
-    prepare_texture(tex);
-  }
-  return (LockedTexture*)&tex;
-}
-
-// Lock the given framebuffer's color attachment to prevent modification.
-LockedTexture* LockFramebuffer(GLuint fboId) {
-  Framebuffer& fb = ctx->framebuffers[fboId];
-  // Only allow locking a framebuffer if it has a valid color attachment.
-  if (!fb.color_attachment) {
-    assert(fb.color_attachment != 0);
-    return nullptr;
-  }
-  return LockTexture(fb.color_attachment);
-}
-
-// Reference an already locked resource
-void LockResource(LockedTexture* resource) {
-  if (!resource) {
-    return;
-  }
-  __sync_fetch_and_add(&resource->locked, 1);
-}
-
-// Remove a lock on a texture that has been previously locked
-void UnlockResource(LockedTexture* resource) {
-  if (!resource) {
-    return;
-  }
-  if (__sync_fetch_and_add(&resource->locked, -1) <= 0) {
-    // The lock should always be non-zero before unlocking.
-    assert(0);
-  }
-}
-
 // Get the underlying buffer for a locked resource
 void* GetResourceBuffer(LockedTexture* resource, int32_t* width,
                         int32_t* height, int32_t* stride) {
@@ -588,6 +576,25 @@ void Composite(LockedTexture* lockedDst, LockedTexture* lockedSrc, GLint srcX,
   }
 }
 
+// Extension used by the SWGL compositor to apply an alpha mask
+// to a texture. The textures must be the same size. The mask
+// must be R8, the texture must be RGBA8.
+void ApplyMask(LockedTexture* lockedDst, LockedTexture* lockedMask) {
+  assert(lockedDst);
+  assert(lockedMask);
+
+  Texture& masktex = *lockedMask;
+  Texture& dsttex = *lockedDst;
+
+  assert(masktex.bpp() == 1);
+  assert(dsttex.bpp() == 4);
+
+  assert(masktex.width == dsttex.width);
+  assert(masktex.height == dsttex.height);
+
+  mask_blit(masktex, dsttex);
+}
+
 }  // extern "C"
 
 // Saturated add helper for YUV conversion. Supported platforms have intrinsics
