@@ -8,7 +8,7 @@
 
 namespace glsl {
 
-enum TextureFormat { RGBA32F, RGBA32I, RGBA8, R8, RG8, R16, RG16, YUV422 };
+enum TextureFormat { RGBA32F, RGBA32I, RGBA8, R8, RG8, R16, RG16, YUY2 };
 
 enum TextureFilter { NEAREST, LINEAR };
 
@@ -106,6 +106,7 @@ float force_scalar(Float f) { return f[0]; }
 
 int32_t force_scalar(I32 i) { return i[0]; }
 
+struct vec3;
 struct vec4;
 struct ivec2;
 
@@ -753,6 +754,8 @@ Float fract(Float v) { return v - floor(v); }
 
 vec2 fract(vec2 v) { return vec2(fract(v.x), fract(v.y)); }
 
+vec2_scalar fract(vec2_scalar v) { return vec2_scalar(fract(v.x), fract(v.y)); }
+
 // X derivatives can be approximated by dFdx(x) = x[1] - x[0].
 // Y derivatives are not easily available since we operate in terms of X spans
 // only. To work around, assume dFdy(p.x) = dFdx(p.y), which only holds for
@@ -2019,6 +2022,14 @@ struct vec4 {
     return *this;
   }
 
+  friend I32 operator==(const vec4& l, const vec4& r) {
+    return l.x == r.x && l.y == r.y && l.z == r.z && l.w == r.w;
+  }
+
+  friend I32 operator!=(const vec4& l, const vec4& r) {
+    return l.x != r.x || l.y != r.y || l.z != r.z || l.w != r.w;
+  }
+
   Float x;
   Float y;
   Float z;
@@ -2285,6 +2296,12 @@ bvec2_scalar notEqual(vec2_scalar x, vec2_scalar y) {
   return bvec2_scalar(notEqual(x.x, y.x), notEqual(x.y, y.y));
 }
 
+vec3 floor(vec3 v) { return vec3(floor(v.x), floor(v.y), floor(v.z)); }
+
+vec4 floor(vec4 v) {
+  return vec4(floor(v.x), floor(v.y), floor(v.z), floor(v.w));
+}
+
 struct mat4_scalar;
 
 struct mat2_scalar {
