@@ -4,11 +4,10 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 452498;
-var summary = 'TM: upvar2 regression tests';
-var actual = '';
-var expect = '';
-
+var BUGNUMBER = 452724;
+var summary = 'Do not crash with JIT: @TraceRecorder::getThis';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
 //-----------------------------------------------------------------------------
 test();
@@ -19,18 +18,9 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
 
-// ------- Comment #112 From Jesse Ruderman
 
-  expect = 'ReferenceError: can\'t access lexical declaration \'q\' before initialization';
+  for (var j=0;j<5;++j) { (0/0) in this; }
 
-  try
-  {
-    q = new Function("(function() { q(3); })(); const q = undefined;"); q();
-  }
-  catch(ex)
-  {
-    actual = ex + '';
-  }
 
   reportCompare(expect, actual, summary);
 }
