@@ -8,11 +8,12 @@
 #include "CryptoTask.h"
 #include "ExtendedValidation.h"
 #include "NSSCertDBTrustDomain.h"
-#include "SharedSSLState.h"
 #include "certdb.h"
+#include "mozilla/glean/SecurityCertverifierMetrics.h"
 #include "mozilla/Assertions.h"
 #include "mozilla/Base64.h"
 #include "mozilla/Casting.h"
+#include "mozilla/Logging.h"
 #include "mozilla/Services.h"
 #include "mozilla/Unused.h"
 #include "mozpkix/Time.h"
@@ -26,8 +27,6 @@
 #include "nsIFile.h"
 #include "nsIMutableArray.h"
 #include "nsIObserverService.h"
-#include "nsIPrefBranch.h"
-#include "nsIPrefService.h"
 #include "nsIPrompt.h"
 #include "nsNSSCertHelper.h"
 #include "nsNSSCertTrust.h"
@@ -45,18 +44,39 @@
 #include "secerr.h"
 #include "ssl.h"
 
+#ifdef MOZ_WIDGET_ANDROID
+#  include "mozilla/java/ClientAuthCertificateManagerWrappers.h"
+#endif
+
 #ifdef XP_WIN
 #  include <winsock.h>  // for ntohl
 #endif
 
 using namespace mozilla;
 using namespace mozilla::psm;
-using mozilla::psm::SharedSSLState;
 
 extern LazyLogModule gPIPNSSLog;
 
 NS_IMPL_ISUPPORTS(nsNSSCertificateDB, nsIX509CertDB)
 
+NS_IMETHODIMP
+nsNSSCertificateDB::CountTrustObjects(uint32_t* aCount) {
+  UniquePK11SlotInfo slot(PK11_GetInternalKeySlot());
+  PK11GenericObject* objects =
+      PK11_FindGenericObjects(slot.get(), CKO_NSS_TRUST);
+  int count = 0;
+  for (PK11GenericObject* cursor = objects; cursor;
+       cursor = PK11_GetNextGenericObject(cursor)) {
+    count++;
+  }
+  PK11_DestroyGenericObjects(objects);
+
+  mozilla::glean::cert_verifier::trust_obj_count.Set(count);
+
+  *aCount = count;
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsNSSCertificateDB::FindCertByDBKey(const nsACString& aDBKey,
                                     /*out*/ nsIX509Cert** _cert) {
@@ -67,7 +87,7 @@ nsNSSCertificateDB::FindCertByDBKey(const nsACString& aDBKey,
     return NS_ERROR_INVALID_ARG;
   }
 
-  nsresult rv = BlockUntilLoadableRootsLoaded();
+  nsresult rv = BlockUntilLoadableCertsLoaded();
   if (NS_FAILED(rv)) {
     return rv;
   }
@@ -81,10 +101,7 @@ nsNSSCertificateDB::FindCertByDBKey(const nsACString& aDBKey,
   if (!cert) {
     return NS_OK;
   }
-  nsCOMPtr<nsIX509Cert> nssCert = nsNSSCertificate::Create(cert.get());
-  if (!nssCert) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
+  nsCOMPtr<nsIX509Cert> nssCert = new nsNSSCertificate(cert.get());
   nssCert.forget(_cert);
   return NS_OK;
 }
@@ -140,48 +157,32 @@ nsresult nsNSSCertificateDB::FindCertByDBKey(const nsACString& aDBKey,
   reader += issuerLen;
   MOZ_ASSERT(reader == decoded.EndReading());
 
+  AutoSearchingForClientAuthCertificates _;
   cert.reset(CERT_FindCertByIssuerAndSN(CERT_GetDefaultCertDB(), &issuerSN));
   return NS_OK;
 }
 
 SECStatus collect_certs(void* arg, SECItem** certs, int numcerts) {
-  CERTDERCerts* collectArgs;
-  SECItem* cert;
-  SECStatus rv;
-
-  collectArgs = (CERTDERCerts*)arg;
-
-  collectArgs->numcerts = numcerts;
-  collectArgs->rawCerts = (SECItem*)PORT_ArenaZAlloc(
-      collectArgs->arena, sizeof(SECItem) * numcerts);
-  if (!collectArgs->rawCerts) return (SECFailure);
-
-  cert = collectArgs->rawCerts;
+  nsTArray<nsTArray<uint8_t>>* certsArray =
+      reinterpret_cast<nsTArray<nsTArray<uint8_t>>*>(arg);
 
   while (numcerts--) {
-    rv = SECITEM_CopyItem(collectArgs->arena, cert, *certs);
-    if (rv == SECFailure) return (SECFailure);
-    cert++;
+    nsTArray<uint8_t> certArray;
+    SECItem* cert = *certs;
+    certArray.AppendElements(cert->data, cert->len);
+    certsArray->AppendElement(std::move(certArray));
     certs++;
   }
-
   return (SECSuccess);
 }
 
-CERTDERCerts* nsNSSCertificateDB::getCertsFromPackage(
-    const UniquePLArenaPool& arena, uint8_t* data, uint32_t length) {
-  CERTDERCerts* collectArgs = PORT_ArenaZNew(arena.get(), CERTDERCerts);
-  if (!collectArgs) {
-    return nullptr;
-  }
-
-  collectArgs->arena = arena.get();
+nsresult nsNSSCertificateDB::getCertsFromPackage(
+    nsTArray<nsTArray<uint8_t>>& collectArgs, uint8_t* data, uint32_t length) {
   if (CERT_DecodeCertPackage(BitwiseCast<char*, uint8_t*>(data), length,
-                             collect_certs, collectArgs) != SECSuccess) {
-    return nullptr;
+                             collect_certs, &collectArgs) != SECSuccess) {
+    return NS_ERROR_FAILURE;
   }
-
-  return collectArgs;
+  return NS_OK;
 }
 
 // When using the sql-backed softoken, trust settings are authenticated using a
@@ -194,11 +195,25 @@ SECStatus ChangeCertTrustWithPossibleAuthentication(
     PR_SetError(SEC_ERROR_LIBRARY_FAILURE, 0);
     return SECFailure;
   }
+
+  RefPtr<SharedCertVerifier> certVerifier(GetDefaultCertVerifier());
+  if (!certVerifier) {
+    PR_SetError(SEC_ERROR_LIBRARY_FAILURE, 0);
+    return SECFailure;
+  }
+
   // NSS ignores the first argument to CERT_ChangeCertTrust
   SECStatus srv = CERT_ChangeCertTrust(nullptr, cert.get(), &trust);
-  if (srv == SECSuccess || PR_GetError() != SEC_ERROR_TOKEN_NOT_LOGGED_IN) {
-    return srv;
+  if (srv != SECSuccess && PR_GetError() != SEC_ERROR_TOKEN_NOT_LOGGED_IN) {
+    return SECFailure;
   }
+  if (srv == SECSuccess) {
+    certVerifier->ClearTrustCache();
+    return SECSuccess;
+  }
+
+  // CERT_ChangeCertTrust failed with SEC_ERROR_TOKEN_NOT_LOGGED_IN, so
+  // authenticate and try again.
   if (cert->slot) {
     // If this certificate is on an external PKCS#11 token, we have to
     // authenticate to that token.
@@ -211,7 +226,13 @@ SECStatus ChangeCertTrustWithPossibleAuthentication(
   if (srv != SECSuccess) {
     return srv;
   }
-  return CERT_ChangeCertTrust(nullptr, cert.get(), &trust);
+  srv = CERT_ChangeCertTrust(nullptr, cert.get(), &trust);
+  if (srv != SECSuccess) {
+    return srv;
+  }
+
+  certVerifier->ClearTrustCache();
+  return SECSuccess;
 }
 
 static nsresult ImportCertsIntoPermanentStorage(
@@ -397,6 +418,21 @@ nsresult nsNSSCertificateDB::handleCACertDownload(NotNull<nsIArray*> x509Certs,
   return ImportCertsIntoPermanentStorage(certList);
 }
 
+nsresult nsNSSCertificateDB::ConstructCertArrayFromUniqueCertList(
+    const UniqueCERTCertList& aCertListIn,
+    nsTArray<RefPtr<nsIX509Cert>>& aCertListOut) {
+  if (!aCertListIn.get()) {
+    return NS_ERROR_INVALID_ARG;
+  }
+
+  for (CERTCertListNode* node = CERT_LIST_HEAD(aCertListIn.get());
+       !CERT_LIST_END(node, aCertListIn.get()); node = CERT_LIST_NEXT(node)) {
+    RefPtr<nsIX509Cert> cert = new nsNSSCertificate(node->cert);
+    aCertListOut.AppendElement(cert);
+  }
+  return NS_OK;
+}
+
 NS_IMETHODIMP
 nsNSSCertificateDB::ImportCertificates(uint8_t* data, uint32_t length,
                                        uint32_t type,
@@ -406,14 +442,10 @@ nsNSSCertificateDB::ImportCertificates(uint8_t* data, uint32_t length,
     return NS_ERROR_FAILURE;
   }
 
-  UniquePLArenaPool arena(PORT_NewArena(DER_DEFAULT_CHUNKSIZE));
-  if (!arena) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
-
-  CERTDERCerts* certCollection = getCertsFromPackage(arena, data, length);
-  if (!certCollection) {
-    return NS_ERROR_FAILURE;
+  nsTArray<nsTArray<uint8_t>> certsArray;
+  nsresult rv = getCertsFromPackage(certsArray, data, length);
+  if (NS_FAILED(rv)) {
+    return rv;
   }
 
   nsCOMPtr<nsIMutableArray> array = nsArrayBase::Create();
@@ -422,13 +454,8 @@ nsNSSCertificateDB::ImportCertificates(uint8_t* data, uint32_t length,
   }
 
   // Now let's create some certs to work with
-  for (int i = 0; i < certCollection->numcerts; i++) {
-    SECItem* currItem = &certCollection->rawCerts[i];
-    nsCOMPtr<nsIX509Cert> cert = nsNSSCertificate::ConstructFromDER(
-        BitwiseCast<char*, unsigned char*>(currItem->data), currItem->len);
-    if (!cert) {
-      return NS_ERROR_FAILURE;
-    }
+  for (nsTArray<uint8_t>& certDER : certsArray) {
+    nsCOMPtr<nsIX509Cert> cert = new nsNSSCertificate(std::move(certDER));
     nsresult rv = array->AppendElement(cert);
     if (NS_FAILED(rv)) {
       return rv;
@@ -441,50 +468,26 @@ nsNSSCertificateDB::ImportCertificates(uint8_t* data, uint32_t length,
 /**
  * Decodes a given array of DER-encoded certificates into temporary storage.
  *
- * @param numcerts
- *        Size of the |certs| array.
  * @param certs
- *        Pointer to array of certs to decode.
+ *        Array in which the decoded certificates are stored as arrays of
+ *        unsigned chars.
  * @param temporaryCerts
  *        List of decoded certificates.
  */
 static nsresult ImportCertsIntoTempStorage(
-    int numcerts, SECItem* certs,
+    nsTArray<nsTArray<uint8_t>>& certs,
     /*out*/ const UniqueCERTCertList& temporaryCerts) {
-  NS_ENSURE_ARG_MIN(numcerts, 1);
-  NS_ENSURE_ARG_POINTER(certs);
   NS_ENSURE_ARG_POINTER(temporaryCerts);
 
-  // CERT_ImportCerts() expects an array of *pointers* to SECItems, so we have
-  // to convert |certs| to such a format first.
-  SECItem** ptrArray =
-      static_cast<SECItem**>(PORT_Alloc(sizeof(SECItem*) * numcerts));
-  if (!ptrArray) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
+  for (nsTArray<uint8_t>& certDER : certs) {
+    CERTCertificate* certificate;
+    SECItem certItem;
+    certItem.len = certDER.Length();
+    certItem.data = certDER.Elements();
+    certificate = CERT_NewTempCertificate(CERT_GetDefaultCertDB(), &certItem,
+                                          nullptr, false, true);
 
-  for (int i = 0; i < numcerts; i++) {
-    ptrArray[i] = &certs[i];
-  }
-
-  CERTCertificate** importedCerts = nullptr;
-  SECStatus srv = CERT_ImportCerts(CERT_GetDefaultCertDB(),
-                                   certUsageAnyCA,  // this argument is ignored
-                                   numcerts, ptrArray, &importedCerts, false,
-                                   false,  // this argument is ignored
-                                   nullptr);
-  PORT_Free(ptrArray);
-  ptrArray = nullptr;
-  if (srv != SECSuccess) {
-    return NS_ERROR_FAILURE;
-  }
-
-  for (int i = 0; i < numcerts; i++) {
-    if (!importedCerts[i]) {
-      continue;
-    }
-
-    UniqueCERTCertificate cert(CERT_DupCertificate(importedCerts[i]));
+    UniqueCERTCertificate cert(certificate);
     if (!cert) {
       continue;
     }
@@ -495,22 +498,17 @@ static nsresult ImportCertsIntoTempStorage(
     }
   }
 
-  CERT_DestroyCertArray(importedCerts, numcerts);
-
   return NS_OK;
 }
 
 NS_IMETHODIMP
 nsNSSCertificateDB::ImportEmailCertificate(uint8_t* data, uint32_t length,
                                            nsIInterfaceRequestor* ctx) {
-  UniquePLArenaPool arena(PORT_NewArena(DER_DEFAULT_CHUNKSIZE));
-  if (!arena) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
+  nsTArray<nsTArray<uint8_t>> certsArray;
 
-  CERTDERCerts* certCollection = getCertsFromPackage(arena, data, length);
-  if (!certCollection) {
-    return NS_ERROR_FAILURE;
+  nsresult rv = getCertsFromPackage(certsArray, data, length);
+  if (NS_FAILED(rv)) {
+    return rv;
   }
 
   UniqueCERTCertList temporaryCerts(CERT_NewCertList());
@@ -518,8 +516,7 @@ nsNSSCertificateDB::ImportEmailCertificate(uint8_t* data, uint32_t length,
     return NS_ERROR_FAILURE;
   }
 
-  nsresult rv = ImportCertsIntoTempStorage(
-      certCollection->numcerts, certCollection->rawCerts, temporaryCerts);
+  rv = ImportCertsIntoTempStorage(certsArray, temporaryCerts);
   if (NS_FAILED(rv)) {
     return rv;
   }
@@ -527,14 +524,14 @@ nsNSSCertificateDB::ImportEmailCertificate(uint8_t* data, uint32_t length,
   return ImportCertsIntoPermanentStorage(temporaryCerts);
 }
 
-nsresult nsNSSCertificateDB::ImportCACerts(int numCACerts, SECItem* caCerts,
+nsresult nsNSSCertificateDB::ImportCACerts(nsTArray<nsTArray<uint8_t>>& caCerts,
                                            nsIInterfaceRequestor* ctx) {
   UniqueCERTCertList temporaryCerts(CERT_NewCertList());
   if (!temporaryCerts) {
     return NS_ERROR_FAILURE;
   }
 
-  nsresult rv = ImportCertsIntoTempStorage(numCACerts, caCerts, temporaryCerts);
+  nsresult rv = ImportCertsIntoTempStorage(caCerts, temporaryCerts);
   if (NS_FAILED(rv)) {
     return rv;
   }
@@ -579,25 +576,31 @@ nsNSSCertificateDB::ImportUserCertificate(uint8_t* data, uint32_t length,
     return NS_ERROR_NOT_SAME_THREAD;
   }
 
-  UniquePLArenaPool arena(PORT_NewArena(DER_DEFAULT_CHUNKSIZE));
-  if (!arena) {
-    return NS_ERROR_OUT_OF_MEMORY;
+  nsTArray<nsTArray<uint8_t>> certsArray;
+
+  nsresult rv = getCertsFromPackage(certsArray, data, length);
+  if (NS_FAILED(rv)) {
+    return rv;
   }
 
-  CERTDERCerts* collectArgs = getCertsFromPackage(arena, data, length);
-  if (!collectArgs) {
-    return NS_ERROR_FAILURE;
+  SECItem certItem;
+
+  if (certsArray.IsEmpty()) {
+    return NS_OK;
   }
 
+  certItem.len = certsArray.ElementAt(0).Length();
+  certItem.data = certsArray.ElementAt(0).Elements();
+
   UniqueCERTCertificate cert(CERT_NewTempCertificate(
-      CERT_GetDefaultCertDB(), collectArgs->rawCerts, nullptr, false, true));
+      CERT_GetDefaultCertDB(), &certItem, nullptr, false, true));
   if (!cert) {
     return NS_ERROR_FAILURE;
   }
 
   UniquePK11SlotInfo slot(PK11_KeyForCertExists(cert.get(), nullptr, ctx));
   if (!slot) {
-    nsCOMPtr<nsIX509Cert> certToShow = nsNSSCertificate::Create(cert.get());
+    nsCOMPtr<nsIX509Cert> certToShow = new nsNSSCertificate(cert.get());
     DisplayCertificateAlert(ctx, "UserCertIgnoredNoPrivateKey", certToShow);
     return NS_ERROR_FAILURE;
   }
@@ -619,15 +622,14 @@ nsNSSCertificateDB::ImportUserCertificate(uint8_t* data, uint32_t length,
   slot = nullptr;
 
   {
-    nsCOMPtr<nsIX509Cert> certToShow = nsNSSCertificate::Create(cert.get());
+    nsCOMPtr<nsIX509Cert> certToShow = new nsNSSCertificate(cert.get());
     DisplayCertificateAlert(ctx, "UserCertImported", certToShow);
   }
 
-  nsresult rv = NS_OK;
-  int numCACerts = collectArgs->numcerts - 1;
-  if (numCACerts) {
-    SECItem* caCerts = collectArgs->rawCerts + 1;
-    rv = ImportCACerts(numCACerts, caCerts, ctx);
+  rv = NS_OK;
+  if (!certsArray.IsEmpty()) {
+    certsArray.RemoveElementAt(0);
+    rv = ImportCACerts(certsArray, ctx);
   }
 
   nsCOMPtr<nsIObserverService> observerService =
@@ -647,28 +649,42 @@ nsNSSCertificateDB::DeleteCertificate(nsIX509Cert* aCert) {
   if (!cert) {
     return NS_ERROR_FAILURE;
   }
-  SECStatus srv = SECSuccess;
 
-  uint32_t certType;
-  aCert->GetCertType(&certType);
-  if (NS_FAILED(aCert->MarkForPermDeletion())) {
-    return NS_ERROR_FAILURE;
+  // Temporary certificates aren't on a slot and will go away when the
+  // nsIX509Cert is destructed.
+  if (cert->slot) {
+    uint32_t certType;
+    nsresult rv = aCert->GetCertType(&certType);
+    if (NS_WARN_IF(NS_FAILED(rv))) {
+      return rv;
+    }
+    if (certType == nsIX509Cert::USER_CERT) {
+      SECStatus srv = PK11_Authenticate(cert->slot, true, nullptr);
+      if (srv != SECSuccess) {
+        return NS_ERROR_FAILURE;
+      }
+      srv = PK11_DeleteTokenCertAndKey(cert.get(), nullptr);
+      if (srv != SECSuccess) {
+        return NS_ERROR_FAILURE;
+      }
+    } else {
+      // For certificates that can't be deleted (e.g. built-in roots), un-set
+      // all trust bits.
+      nsNSSCertTrust trust(0, 0);
+      SECStatus srv = ChangeCertTrustWithPossibleAuthentication(
+          cert, trust.GetTrust(), nullptr);
+      if (srv != SECSuccess) {
+        return NS_ERROR_FAILURE;
+      }
+      if (!PK11_IsReadOnly(cert->slot)) {
+        srv = SEC_DeletePermCertificate(cert.get());
+        if (srv != SECSuccess) {
+          return NS_ERROR_FAILURE;
+        }
+      }
+    }
   }
 
-  if (cert->slot && certType != nsIX509Cert::USER_CERT) {
-    // To delete a cert of a slot (builtin, most likely), mark it as
-    // completely untrusted.  This way we keep a copy cached in the
-    // local database, and next time we try to load it off of the
-    // external token/slot, we'll know not to trust it.  We don't
-    // want to do that with user certs, because a user may  re-store
-    // the cert onto the card again at which point we *will* want to
-    // trust that cert if it chains up properly.
-    nsNSSCertTrust trust(0, 0);
-    srv = ChangeCertTrustWithPossibleAuthentication(cert, trust.GetTrust(),
-                                                    nullptr);
-  }
-  MOZ_LOG(gPIPNSSLog, LogLevel::Debug, ("cert deleted: %d", srv));
-
   nsCOMPtr<nsIObserverService> observerService =
       mozilla::services::GetObserverService();
   if (observerService) {
@@ -676,7 +692,7 @@ nsNSSCertificateDB::DeleteCertificate(nsIX509Cert* aCert) {
                                      nullptr);
   }
 
-  return (srv) ? NS_ERROR_FAILURE : NS_OK;
+  return NS_OK;
 }
 
 NS_IMETHODIMP
@@ -715,7 +731,7 @@ nsNSSCertificateDB::IsCertTrusted(nsIX509Cert* cert, uint32_t certType,
   NS_ENSURE_ARG_POINTER(_isTrusted);
   *_isTrusted = false;
 
-  nsresult rv = BlockUntilLoadableRootsLoaded();
+  nsresult rv = BlockUntilLoadableCertsLoaded();
   if (NS_FAILED(rv)) {
     return rv;
   }
@@ -816,7 +832,7 @@ nsNSSCertificateDB::ImportPKCS12File(nsIFile* aFile, const nsAString& aPassword,
   if (!NS_IsMainThread()) {
     return NS_ERROR_NOT_SAME_THREAD;
   }
-  nsresult rv = BlockUntilLoadableRootsLoaded();
+  nsresult rv = BlockUntilLoadableCertsLoaded();
   if (NS_FAILED(rv)) {
     return rv;
   }
@@ -835,24 +851,23 @@ nsNSSCertificateDB::ImportPKCS12File(nsIFile* aFile, const nsAString& aPassword,
 }
 
 NS_IMETHODIMP
-nsNSSCertificateDB::ExportPKCS12File(nsIFile* aFile, uint32_t aCount,
-                                     nsIX509Cert** aCerts,
-                                     const nsAString& aPassword,
-                                     uint32_t* aError) {
+nsNSSCertificateDB::ExportPKCS12File(
+    nsIFile* aFile, const nsTArray<RefPtr<nsIX509Cert>>& aCerts,
+    const nsAString& aPassword, uint32_t* aError) {
   if (!NS_IsMainThread()) {
     return NS_ERROR_NOT_SAME_THREAD;
   }
-  nsresult rv = BlockUntilLoadableRootsLoaded();
+  nsresult rv = BlockUntilLoadableCertsLoaded();
   if (NS_FAILED(rv)) {
     return rv;
   }
 
   NS_ENSURE_ARG(aFile);
-  if (aCount == 0) {
+  if (aCerts.IsEmpty()) {
     return NS_OK;
   }
   nsPKCS12Blob blob;
-  return blob.ExportToFile(aFile, aCerts, aCount, aPassword, *aError);
+  return blob.ExportToFile(aFile, aCerts, aPassword, *aError);
 }
 
 NS_IMETHODIMP
@@ -875,21 +890,31 @@ nsNSSCertificateDB::ConstructX509FromBase64(const nsACString& base64,
     return rv;
   }
 
-  return ConstructX509(certDER, _retval);
+  return ConstructX509FromSpan(AsBytes(Span(certDER)), _retval);
 }
 
 NS_IMETHODIMP
-nsNSSCertificateDB::ConstructX509(const nsACString& certDER,
+nsNSSCertificateDB::ConstructX509(const nsTArray<uint8_t>& certDER,
                                   nsIX509Cert** _retval) {
+  return ConstructX509FromSpan(Span(certDER.Elements(), certDER.Length()),
+                               _retval);
+}
+
+nsresult nsNSSCertificateDB::ConstructX509FromSpan(
+    Span<const uint8_t> aInputSpan, nsIX509Cert** _retval) {
   if (NS_WARN_IF(!_retval)) {
     return NS_ERROR_INVALID_POINTER;
   }
 
+  if (aInputSpan.Length() > std::numeric_limits<unsigned int>::max()) {
+    return NS_ERROR_ILLEGAL_VALUE;
+  }
+
   SECItem certData;
   certData.type = siDERCertBuffer;
-  certData.data =
-      BitwiseCast<unsigned char*, const char*>(certDER.BeginReading());
-  certData.len = certDER.Length();
+  certData.data = const_cast<unsigned char*>(
+      reinterpret_cast<const unsigned char*>(aInputSpan.Elements()));
+  certData.len = aInputSpan.Length();
 
   UniqueCERTCertificate cert(CERT_NewTempCertificate(
       CERT_GetDefaultCertDB(), &certData, nullptr, false, true));
@@ -897,10 +922,7 @@ nsNSSCertificateDB::ConstructX509(const nsACString& certDER,
     return (PORT_GetError() == SEC_ERROR_NO_MEMORY) ? NS_ERROR_OUT_OF_MEMORY
                                                     : NS_ERROR_FAILURE;
 
-  nsCOMPtr<nsIX509Cert> nssCert = nsNSSCertificate::Create(cert.get());
-  if (!nssCert) {
-    return NS_ERROR_OUT_OF_MEMORY;
-  }
+  nsCOMPtr<nsIX509Cert> nssCert = new nsNSSCertificate(cert.get());
   nssCert.forget(_retval);
   return NS_OK;
 }
@@ -912,7 +934,7 @@ void nsNSSCertificateDB::get_default_nickname(CERTCertificate* cert,
 
   CK_OBJECT_HANDLE keyHandle;
 
-  if (NS_FAILED(BlockUntilLoadableRootsLoaded())) {
+  if (NS_FAILED(BlockUntilLoadableCertsLoaded())) {
     return;
   }
 
@@ -1003,6 +1025,25 @@ NS_IMETHODIMP
 nsNSSCertificateDB::AddCertFromBase64(const nsACString& aBase64,
                                       const nsACString& aTrust,
                                       nsIX509Cert** addedCertificate) {
+  // Base64Decode() doesn't consider a zero length input as an error, and just
+  // returns the empty string. We don't want this behavior, so the below check
+  // catches this case.
+  if (aBase64.Length() < 1) {
+    return NS_ERROR_ILLEGAL_VALUE;
+  }
+
+  nsAutoCString aCertDER;
+  nsresult rv = Base64Decode(aBase64, aCertDER);
+  if (NS_FAILED(rv)) {
+    return rv;
+  }
+  return AddCert(aCertDER, aTrust, addedCertificate);
+}
+
+NS_IMETHODIMP
+nsNSSCertificateDB::AddCert(const nsACString& aCertDER,
+                            const nsACString& aTrust,
+                            nsIX509Cert** addedCertificate) {
   MOZ_ASSERT(addedCertificate);
   if (!addedCertificate) {
     return NS_ERROR_INVALID_ARG;
@@ -1016,7 +1057,8 @@ nsNSSCertificateDB::AddCertFromBase64(const nsACString& aBase64,
   }
 
   nsCOMPtr<nsIX509Cert> newCert;
-  nsresult rv = ConstructX509FromBase64(aBase64, getter_AddRefs(newCert));
+  nsresult rv =
+      ConstructX509FromSpan(AsBytes(Span(aCertDER)), getter_AddRefs(newCert));
   if (NS_FAILED(rv)) {
     return rv;
   }
@@ -1058,16 +1100,6 @@ nsNSSCertificateDB::AddCertFromBase64(const nsACString& aBase64,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsNSSCertificateDB::AddCert(const nsACString& aCertDER,
-                            const nsACString& aTrust,
-                            nsIX509Cert** addedCertificate) {
-  nsCString base64;
-  nsresult rv = Base64Encode(aCertDER, base64);
-  NS_ENSURE_SUCCESS(rv, rv);
-  return AddCertFromBase64(base64, aTrust, addedCertificate);
-}
-
 NS_IMETHODIMP
 nsNSSCertificateDB::SetCertTrustFromString(nsIX509Cert* cert,
                                            const nsACString& trustString) {
@@ -1085,9 +1117,80 @@ nsNSSCertificateDB::SetCertTrustFromString(nsIX509Cert* cert,
   return MapSECStatus(srv);
 }
 
+NS_IMETHODIMP nsNSSCertificateDB::AsPKCS7Blob(
+    const nsTArray<RefPtr<nsIX509Cert>>& certList, nsACString& _retval) {
+  if (certList.IsEmpty()) {
+    return NS_ERROR_INVALID_ARG;
+  }
+
+  UniqueNSSCMSMessage cmsg(NSS_CMSMessage_Create(nullptr));
+  if (!cmsg) {
+    MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
+            ("nsNSSCertificateDB::AsPKCS7Blob - can't create CMS message"));
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+
+  UniqueNSSCMSSignedData sigd(nullptr);
+  for (const auto& cert : certList) {
+    // We need an owning handle when calling nsIX509Cert::GetCert().
+    UniqueCERTCertificate nssCert(cert->GetCert());
+    if (!sigd) {
+      sigd.reset(
+          NSS_CMSSignedData_CreateCertsOnly(cmsg.get(), nssCert.get(), false));
+      if (!sigd) {
+        MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
+                ("nsNSSCertificateDB::AsPKCS7Blob - can't create SignedData"));
+        return NS_ERROR_FAILURE;
+      }
+    } else if (NSS_CMSSignedData_AddCertificate(sigd.get(), nssCert.get()) !=
+               SECSuccess) {
+      MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
+              ("nsNSSCertificateDB::AsPKCS7Blob - can't add cert"));
+      return NS_ERROR_FAILURE;
+    }
+  }
+
+  NSSCMSContentInfo* cinfo = NSS_CMSMessage_GetContentInfo(cmsg.get());
+  if (NSS_CMSContentInfo_SetContent_SignedData(cmsg.get(), cinfo, sigd.get()) !=
+      SECSuccess) {
+    MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
+            ("nsNSSCertificateDB::AsPKCS7Blob - can't attach SignedData"));
+    return NS_ERROR_FAILURE;
+  }
+  // cmsg owns sigd now.
+  Unused << sigd.release();
+
+  UniquePLArenaPool arena(PORT_NewArena(1024));
+  if (!arena) {
+    MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
+            ("nsNSSCertificateDB::AsPKCS7Blob - out of memory"));
+    return NS_ERROR_OUT_OF_MEMORY;
+  }
+
+  SECItem certP7 = {siBuffer, nullptr, 0};
+  NSSCMSEncoderContext* ecx = NSS_CMSEncoder_Start(
+      cmsg.get(), nullptr, nullptr, &certP7, arena.get(), nullptr, nullptr,
+      nullptr, nullptr, nullptr, nullptr);
+  if (!ecx) {
+    MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
+            ("nsNSSCertificateDB::AsPKCS7Blob - can't create encoder"));
+    return NS_ERROR_FAILURE;
+  }
+
+  if (NSS_CMSEncoder_Finish(ecx) != SECSuccess) {
+    MOZ_LOG(gPIPNSSLog, LogLevel::Debug,
+            ("nsNSSCertificateDB::AsPKCS7Blob - failed to add encoded data"));
+    return NS_ERROR_FAILURE;
+  }
+
+  _retval.Assign(nsDependentCSubstring(
+      reinterpret_cast<const char*>(certP7.data), certP7.len));
+  return NS_OK;
+}
+
 NS_IMETHODIMP
-nsNSSCertificateDB::GetCerts(nsIX509CertList** _retval) {
-  nsresult rv = BlockUntilLoadableRootsLoaded();
+nsNSSCertificateDB::GetCerts(nsTArray<RefPtr<nsIX509Cert>>& _retval) {
+  nsresult rv = BlockUntilLoadableCertsLoaded();
   if (NS_FAILED(rv)) {
     return rv;
   }
@@ -1098,84 +1201,111 @@ nsNSSCertificateDB::GetCerts(nsIX509CertList** _retval) {
   }
 
   nsCOMPtr<nsIInterfaceRequestor> ctx = new PipUIContext();
-  nsCOMPtr<nsIX509CertList> nssCertList;
+  AutoSearchingForClientAuthCertificates _;
   UniqueCERTCertList certList(PK11_ListCerts(PK11CertListUnique, ctx));
-
-  // nsNSSCertList 1) adopts certList, and 2) handles the nullptr case fine.
-  // (returns an empty list)
-  nssCertList = new nsNSSCertList(std::move(certList));
-
-  nssCertList.forget(_retval);
-  return NS_OK;
+  if (!certList) {
+    return NS_ERROR_FAILURE;
+  }
+  return nsNSSCertificateDB::ConstructCertArrayFromUniqueCertList(certList,
+                                                                  _retval);
 }
 
-nsresult VerifyCertAtTime(nsIX509Cert* aCert,
-                          int64_t /*SECCertificateUsage*/ aUsage,
+static mozilla::Result<VerifyUsage, nsresult> MapX509UsageToVerifierUsage(
+    nsIX509CertDB::VerifyUsage usage) {
+  switch (usage) {
+    case nsIX509CertDB::verifyUsageTLSServer:
+      return VerifyUsage::TLSServer;
+    case nsIX509CertDB::verifyUsageTLSServerCA:
+      return VerifyUsage::TLSServerCA;
+    case nsIX509CertDB::verifyUsageTLSClient:
+      return VerifyUsage::TLSClient;
+    case nsIX509CertDB::verifyUsageTLSClientCA:
+      return VerifyUsage::TLSClientCA;
+    case nsIX509CertDB::verifyUsageEmailSigner:
+      return VerifyUsage::EmailSigner;
+    case nsIX509CertDB::verifyUsageEmailRecipient:
+      return VerifyUsage::EmailRecipient;
+    case nsIX509CertDB::verifyUsageEmailCA:
+      return VerifyUsage::EmailCA;
+  }
+  return Err(NS_ERROR_INVALID_ARG);
+}
+
+nsresult VerifyCertAtTime(nsIX509Cert* aCert, nsIX509CertDB::VerifyUsage aUsage,
                           uint32_t aFlags, const nsACString& aHostname,
                           mozilla::pkix::Time aTime,
-                          nsIX509CertList** aVerifiedChain, bool* aHasEVPolicy,
+                          nsTArray<RefPtr<nsIX509Cert>>& aVerifiedChain,
+                          bool* aHasEVPolicy,
                           int32_t* /*PRErrorCode*/ _retval) {
   NS_ENSURE_ARG_POINTER(aCert);
   NS_ENSURE_ARG_POINTER(aHasEVPolicy);
-  NS_ENSURE_ARG_POINTER(aVerifiedChain);
   NS_ENSURE_ARG_POINTER(_retval);
 
-  *aVerifiedChain = nullptr;
-  *aHasEVPolicy = false;
-  *_retval = PR_UNKNOWN_ERROR;
-
-  UniqueCERTCertificate nssCert(aCert->GetCert());
-  if (!nssCert) {
+  if (!aVerifiedChain.IsEmpty()) {
     return NS_ERROR_INVALID_ARG;
   }
 
+  *aHasEVPolicy = false;
+  *_retval = PR_UNKNOWN_ERROR;
+
   RefPtr<SharedCertVerifier> certVerifier(GetDefaultCertVerifier());
   NS_ENSURE_TRUE(certVerifier, NS_ERROR_FAILURE);
 
-  UniqueCERTCertList resultChain;
-  SECOidTag evOidPolicy;
+  nsTArray<nsTArray<uint8_t>> resultChain;
+  EVStatus evStatus;
   mozilla::pkix::Result result;
 
-  if (!aHostname.IsVoid() && aUsage == certificateUsageSSLServer) {
-    result = certVerifier->VerifySSLServerCert(
-        nssCert,
-        nullptr,  // stapledOCSPResponse
-        nullptr,  // sctsFromTLSExtension
-        aTime,
-        nullptr,  // Assume no context
-        aHostname, resultChain,
-        false,  // don't save intermediates
-        aFlags, OriginAttributes(), &evOidPolicy);
+  nsTArray<uint8_t> certBytes;
+  nsresult nsrv = aCert->GetRawDER(certBytes);
+  if (NS_FAILED(nsrv)) {
+    return nsrv;
+  }
+
+  if (!aHostname.IsVoid() && aUsage == nsIX509CertDB::verifyUsageTLSServer) {
+    result =
+        certVerifier->VerifySSLServerCert(certBytes, aTime,
+                                          nullptr,  // Assume no context
+                                          aHostname, resultChain, aFlags,
+                                          Nothing(),  // extraCertificates
+                                          Nothing(),  // stapledOCSPResponse
+                                          Nothing(),  // sctsFromTLSExtension
+                                          Nothing(),  // dcInfo
+                                          OriginAttributes(), &evStatus);
   } else {
     const nsCString& flatHostname = PromiseFlatCString(aHostname);
+    VerifyUsage vu;
+    MOZ_TRY_VAR(vu, MapX509UsageToVerifierUsage(aUsage));
     result = certVerifier->VerifyCert(
-        nssCert.get(), aUsage, aTime,
+        certBytes, vu, aTime,
         nullptr,  // Assume no context
         aHostname.IsVoid() ? nullptr : flatHostname.get(), resultChain, aFlags,
-        nullptr,  // stapledOCSPResponse
-        nullptr,  // sctsFromTLSExtension
-        OriginAttributes(), &evOidPolicy);
+        Nothing(),  // extraCertificates
+        Nothing(),  // stapledOCSPResponse
+        Nothing(),  // sctsFromTLSExtension
+        OriginAttributes(), &evStatus);
   }
 
-  nsCOMPtr<nsIX509CertList> nssCertList;
-  // This adopts the list
-  nssCertList = new nsNSSCertList(std::move(resultChain));
-  NS_ENSURE_TRUE(nssCertList, NS_ERROR_FAILURE);
+  if (result == mozilla::pkix::Success) {
+    for (auto& certDER : resultChain) {
+      RefPtr<nsIX509Cert> cert = new nsNSSCertificate(std::move(certDER));
+      aVerifiedChain.AppendElement(cert);
+    }
+
+    if (evStatus == EVStatus::EV) {
+      *aHasEVPolicy = true;
+    }
+  }
 
   *_retval = mozilla::pkix::MapResultToPRErrorCode(result);
-  if (result == mozilla::pkix::Success && evOidPolicy != SEC_OID_UNKNOWN) {
-    *aHasEVPolicy = true;
-  }
-  nssCertList.forget(aVerifiedChain);
 
   return NS_OK;
 }
 
 class VerifyCertAtTimeTask final : public CryptoTask {
  public:
-  VerifyCertAtTimeTask(nsIX509Cert* aCert, int64_t aUsage, uint32_t aFlags,
-                       const nsACString& aHostname, uint64_t aTime,
-                       nsICertVerificationCallback* aCallback)
+  VerifyCertAtTimeTask(nsIX509Cert* aCert, nsIX509CertDB::VerifyUsage aUsage,
+                       uint32_t aFlags, const nsACString& aHostname,
+                       uint64_t aTime, nsICertVerificationCallback* aCallback)
       : mCert(aCert),
         mUsage(aUsage),
         mFlags(aFlags),
@@ -1184,7 +1314,6 @@ class VerifyCertAtTimeTask final : public CryptoTask {
         mCallback(new nsMainThreadPtrHolder<nsICertVerificationCallback>(
             "nsICertVerificationCallback", aCallback)),
         mPRErrorCode(SEC_ERROR_LIBRARY_FAILURE),
-        mVerifiedCertList(nullptr),
         mHasEVPolicy(false) {}
 
  private:
@@ -1195,14 +1324,14 @@ class VerifyCertAtTimeTask final : public CryptoTask {
     }
     return VerifyCertAtTime(mCert, mUsage, mFlags, mHostname,
                             mozilla::pkix::TimeFromEpochInSeconds(mTime),
-                            getter_AddRefs(mVerifiedCertList), &mHasEVPolicy,
-                            &mPRErrorCode);
+                            mVerifiedCertList, &mHasEVPolicy, &mPRErrorCode);
   }
 
   virtual void CallCallback(nsresult rv) override {
     if (NS_FAILED(rv)) {
-      Unused << mCallback->VerifyCertFinished(SEC_ERROR_LIBRARY_FAILURE,
-                                              nullptr, false);
+      nsTArray<RefPtr<nsIX509Cert>> tmp;
+      Unused << mCallback->VerifyCertFinished(SEC_ERROR_LIBRARY_FAILURE, tmp,
+                                              false);
     } else {
       Unused << mCallback->VerifyCertFinished(mPRErrorCode, mVerifiedCertList,
                                               mHasEVPolicy);
@@ -1210,24 +1339,24 @@ class VerifyCertAtTimeTask final : public CryptoTask {
   }
 
   nsCOMPtr<nsIX509Cert> mCert;
-  int64_t mUsage;
+  nsIX509CertDB::VerifyUsage mUsage;
   uint32_t mFlags;
   nsCString mHostname;
   uint64_t mTime;
   nsMainThreadPtrHandle<nsICertVerificationCallback> mCallback;
   int32_t mPRErrorCode;
-  nsCOMPtr<nsIX509CertList> mVerifiedCertList;
+  nsTArray<RefPtr<nsIX509Cert>> mVerifiedCertList;
   bool mHasEVPolicy;
 };
 
 NS_IMETHODIMP
 nsNSSCertificateDB::AsyncVerifyCertAtTime(
-    nsIX509Cert* aCert, int64_t /*SECCertificateUsage*/ aUsage, uint32_t aFlags,
+    nsIX509Cert* aCert, nsIX509CertDB::VerifyUsage aUsage, uint32_t aFlags,
     const nsACString& aHostname, uint64_t aTime,
     nsICertVerificationCallback* aCallback) {
   RefPtr<VerifyCertAtTimeTask> task(new VerifyCertAtTimeTask(
       aCert, aUsage, aFlags, aHostname, aTime, aCallback));
-  return task->Dispatch("VerifyCert");
+  return task->Dispatch();
 }
 
 NS_IMETHODIMP
@@ -1237,3 +1366,30 @@ nsNSSCertificateDB::ClearOCSPCache() {
   certVerifier->ClearOCSPCache();
   return NS_OK;
 }
+
+NS_IMETHODIMP
+nsNSSCertificateDB::GetAndroidCertificateFromAlias(
+    const nsAString& aAlias, /*out*/ nsIX509Cert** aResult) {
+  *aResult = nullptr;
+#ifndef MOZ_WIDGET_ANDROID
+  return NS_ERROR_NOT_AVAILABLE;
+#else
+  if (!jni::IsAvailable()) {
+    MOZ_LOG(gPIPNSSLog, LogLevel::Debug, ("JNI not available"));
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+
+  jni::String::LocalRef alias = jni::StringParam(aAlias);
+  jni::ByteArray::LocalRef certificateBytes =
+      java::ClientAuthCertificateManager::GetCertificateFromAlias(alias);
+  if (!certificateBytes) {
+    return NS_ERROR_NOT_AVAILABLE;
+  }
+  nsTArray<uint8_t> certificateByteArray(
+      certificateBytes->GetElements().Elements(), certificateBytes->Length());
+  nsCOMPtr<nsIX509Cert> certificate(
+      new nsNSSCertificate(std::move(certificateByteArray)));
+  certificate.forget(aResult);
+  return NS_OK;
+#endif  // MOZ_WIDGET_ANDROID
+}
